package org.jeecg.modules.ctop.controller;

import cn.com.ctop.common.utils.Check;
import cn.com.ctop.common.utils.HttpUtils;
import cn.com.ctop.common.utils.PropertiesUtils;
import cn.com.ctop.toutiao.common.BytedanceInterfaceConstant;
import com.alibaba.fastjson.JSONObject;
import com.google.gson.Gson;
import constant.KuaishouInterfaceConstant;
import org.apache.commons.lang.StringUtils;
import org.jeecg.modules.ctop.entity.CTopOauthToken;
import org.jeecg.modules.ctop.mapper.CTopOauthTokenMapper;
import org.jeecg.modules.ctop.service.ICTopOauthTokenService;
import org.jeecg.modules.ctop.service.IBindAccountService;
import org.jeecg.modules.kuaishou.service.IKuaishouInterfaceService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.HashMap;
import java.util.Map;
import java.util.UUID;

@Controller
@RequestMapping("/auth")
public class AuthController {
    private static final Logger logger = LoggerFactory.getLogger(AuthController.class);
    @Autowired
    private IKuaishouInterfaceService kuaishouInterfaceService;
    @Autowired
    private IBindAccountService bindAccountService;

    @Autowired
    private CTopOauthTokenMapper cTopOauthTokenMapper;

    @RequestMapping("/begin")
    public String authBegin(HttpServletRequest request,
                            HttpServletResponse response,
                            @RequestParam("auth_type") String authType) {
        try {
            String codeUrl = "";
            switch (authType) {
                case "bytedance":
                    codeUrl = bindAccountService.getByteDanceCodeUrl(UUID.randomUUID().toString());
                    System.out.println(codeUrl);
                    return "redirect:" + codeUrl;
                case "kuaishou":
                    codeUrl = bindAccountService.getKuaishouCodeUrl(UUID.randomUUID().toString());
                    System.out.println(codeUrl);
                    return "redirect:" + codeUrl;
                default:
                    System.out.println("");

            }
        } catch (Exception e) {
            e.printStackTrace();
        }

        return "auth_error";
    }

    @RequestMapping("/bytedance")
    @ResponseBody
    public String bytedance(HttpServletRequest request,
                            HttpServletResponse response,
                            @RequestParam("auth_code") String authCode,
                            @RequestParam("state") String state, @RequestParam("accountId") String accountId) {
        System.out.println(request.getQueryString());
        if (StringUtils.isNotBlank(authCode)) {
            System.out.println(this.getByteDanceAccessToken(authCode, accountId, state));
            return "auth_success";
        }
        return "auth_fail";
    }

    @GetMapping("bytedance/refreshtoken")
    @ResponseBody
    public Map<String, Object> refreshToken(String accountId) {
        return null;
    }

    @Autowired
    private ICTopOauthTokenService tokenService;

    public String getByteDanceAccessToken(String authCode, String accountId, String state) {
        Map<String, Object> param = new HashMap<String, Object>();
        param.put("app_id", PropertiesUtils.getValue("bytedance_config", "bytedance_appid"));
        param.put("secret", PropertiesUtils.getValue("bytedance_config", "bytedance_secret"));
        param.put("grant_type", "auth_code");
        param.put("auth_code", authCode);

        Gson gson = new Gson();
        String result = "";
        try {
            result = HttpUtils.httpPostRequest(PropertiesUtils.getValue("bytedance_config", "bytedance_api_url") + BytedanceInterfaceConstant.AUTH_TOKEN, param, new HashMap<>());
            JSONObject resultObject = JSONObject.parseObject(result);
            Integer code = resultObject.getInteger("code");
            String message = resultObject.getString("message");
            if (null == code || code != 0) {
                logger.info("获取token失败,accountId:{},message:{}", accountId, message);
            }
            JSONObject data = resultObject.getJSONObject("data");
            CTopOauthToken token = new CTopOauthToken(accountId, data);
            if (Check.isNull(token)) {
                Map<String, Object> deleteMap = new HashMap<>();
                deleteMap.put("advertiser_id", state);
                deleteMap.put("account_id", token.getAccountId());
                cTopOauthTokenMapper.deleteByMap(deleteMap); //删除 广告主id下的相关授权信息
                cTopOauthTokenMapper.insert(token);
                bindAccountService.addBindAccount(token.getAccountId(), KuaishouInterfaceConstant.TYPE_AUTHORIZATION, state, KuaishouInterfaceConstant.LOGIN_TYPE_BYTEDANCE); //账号绑定
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return result;
    }
}
