package org.jeecg.modules.ctop.service.impl;

import cn.com.ctop.common.utils.Check;
import cn.com.ctop.common.utils.PropertiesUtils;
import cn.com.ctop.toutiao.common.BytedanceInterfaceConstant;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import constant.KuaishouInterfaceConstant;
import org.jeecg.modules.ctop.entity.BindAccount;
import org.jeecg.modules.ctop.mapper.BindAccountMapper;
import org.jeecg.modules.ctop.service.IBindAccountService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @Description: 账户绑定
 * @Author: jeecg-boot
 * @Date: 2019-07-30
 * @Version: V1.0
 */
@Service
public class BindAccountServiceImpl extends ServiceImpl<BindAccountMapper, BindAccount> implements IBindAccountService {
    private static final Logger logger = LoggerFactory.getLogger(BindAccountServiceImpl.class);

    @Autowired
    BindAccountMapper bindAccountMapper;

    @Override
    public boolean bindLogin(String advertiserId, String accountName, String password, String loginType) {
        Map<String, Object> requestMap = new HashMap<>();
        requestMap.put("advertiser_id", advertiserId);
        requestMap.put("account_name", accountName);
        requestMap.put("login_type", loginType);
        List<BindAccount> bindAccounts = bindAccountMapper.selectByMap(requestMap);
        if (!Check.isNull(bindAccounts)) {
            return false;
        }
        BindAccount bindAccount = new BindAccount();
        bindAccount.setAdvertiserId(advertiserId);
        bindAccount.setAccountName(accountName);
        bindAccount.setPassword(password);
        bindAccount.setType(KuaishouInterfaceConstant.TYPE_LOGIN);
        bindAccount.setLoginType(loginType);
        int i = bindAccountMapper.insert(bindAccount);
        if (i > 0) {
            logger.info("登录账号绑定成功，广告主id:{}", bindAccount.getAdvertiserId());
            return true;
        }

        return false;
    }

    /**
     * 授权
     *
     * @param advertiserId
     * @param authorizationType
     * @return
     */
    @Override
    public String bindAuthorization(String advertiserId, String authorizationType) throws UnsupportedEncodingException {
        String codeUrl = "";
        if ("kuaishou".equals(authorizationType)) {
            codeUrl = getKuaishouCodeUrl(advertiserId);
        }
        if ("toutiao".equals(authorizationType)) {
            codeUrl = getByteDanceCodeUrl(advertiserId);
        }

        return codeUrl;
    }


    /**
     * 授权 绑定账号信息
     *
     * @param accountId
     * @param type
     * @param state
     * @param loginType
     */
    @Override
    public void addBindAccount(Long accountId, String type, String state, String loginType) {
        Map<String, Object> deleteMap = new HashMap<>();
        deleteMap.put("advertiser_id", state);
        deleteMap.put("account_id", accountId);
        deleteMap.put("type", type);
        deleteMap.put("login_type", loginType);
        bindAccountMapper.deleteByMap(deleteMap);
        BindAccount bindAccount = new BindAccount();
        bindAccount.setAccountId(accountId);
        bindAccount.setLoginType(loginType);
        bindAccount.setAdvertiserId(state);
        bindAccount.setType(type);
        int i = bindAccountMapper.insert(bindAccount);
        if (i > 0) {
            logger.info("授权账号绑定成功，本地id:{}，accountId:{}", bindAccount.getAdvertiserId(), accountId);
        }
    }

    @Override
    public String getKuaishouCodeUrl(String state) throws UnsupportedEncodingException {
        StringBuffer sb = new StringBuffer();
        sb.append(PropertiesUtils.getValue("kuaishou_config", "kuaishou_auth_url"))
                .append(KuaishouInterfaceConstant.AUTH_PATH)
                .append("?app_id=" + PropertiesUtils.getValue("kuaishou_config", "kuaishou_appid"))
                .append("&state=" + state)
                .append("&scope=%5B%22report_service%22%2C%22ad_query%22%2C%22ad_manage%22%5D")
                .append("&redirect_uri=" + URLEncoder.encode(PropertiesUtils.getValue("kuaishou_config", "kuaishou_callback_url"), "UTF-8"));
        return sb.toString();
    }


    public String getByteDanceCodeUrl(String state) throws UnsupportedEncodingException {
        StringBuffer sb = new StringBuffer();
        sb.append(PropertiesUtils.getValue("bytedance_config", "bytedance_auth_url"))
                .append(BytedanceInterfaceConstant.AUTH_PATH)
                .append("?app_id=" + PropertiesUtils.getValue("bytedance_config", "bytedance_appid"))
                .append("&state=" + state)
                .append("&redirect_uri=" + URLEncoder.encode(PropertiesUtils.getValue("bytedance_config", "bytedance_callback_url"), "UTF-8"));
        return sb.toString();
    }


}
