package org.jeecg.modules.ctop.service.impl;

import cn.com.ctop.common.utils.PropertiesUtils;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.ContentType;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClientBuilder;
import org.jeecg.modules.ctop.entity.ByteDanceAdvertisePlan;
import org.jeecg.modules.ctop.entity.CTopOauthToken;
import org.jeecg.modules.ctop.mapper.ByteDanceAdvertisePlanMapper;
import org.jeecg.modules.ctop.service.IByteDanceAdvertisePlanService;
import org.jeecg.modules.ctop.service.ICTopOauthTokenService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.HashMap;
import java.util.Map;

/**
 * @Description: 今日头条广告计划信息
 * @Author: jeecg-boot
 * @Date: 2019-07-22
 * @Version: V1.0
 */
@Service
public class ByteDanceAdvertisePlanServiceImpl extends ServiceImpl<ByteDanceAdvertisePlanMapper, ByteDanceAdvertisePlan> implements IByteDanceAdvertisePlanService {
    private static final Logger logger = LoggerFactory.getLogger(ByteDanceAdvertisePlanServiceImpl.class);

    @Override
    public Map<String, Object> advertiserPlanCreate(String accountId, Long campaignId, String dataString) {
        Map<String, Object> resultMap = new HashMap<>();
        //1：获取token
        CTopOauthToken token = tokenService.getOAuthTokenByAccountId(accountId);
        JSONObject data = JSONObject.parseObject(dataString);
        data.put("advertiser_id", token.getAccountId());
        data.put("campaign_id", campaignId);
        JSONObject result = createAd(data, token.getAccessToken());
        Integer code = result.getInteger("code");
        String message = result.getString("message");
        if (null == code || code != 0) {
            logger.info("广告计划创建失败,accountId:{},message:{}", accountId, message);
            resultMap.put("code", -1);
            resultMap.put("message", "广告计划创建失败");
            return resultMap;
        }
        JSONObject dataObject = result.getJSONObject("data");
        Long adId = dataObject.getLong("ad_id");
        ByteDanceAdvertisePlan plan = new ByteDanceAdvertisePlan(dataObject, accountId, adId, data);
        planMapper.insert(plan);
        resultMap.put("code", 0);
        resultMap.put("message", "广告计划创建成功");
        return resultMap;
    }

    @Autowired
    private ByteDanceAdvertisePlanMapper planMapper;

    public JSONObject createAd(JSONObject data, String token) {
        // 请求地址
        String url = PropertiesUtils.getValue("bytedance_config", "bytedance_api_url") + PropertiesUtils.getValue("bytedance_config", "bytedance_v2_ad_create");
        // 构造请求
        HttpPost httpEntity = new HttpPost(url);
        httpEntity.setHeader("Access-Token", token);
        CloseableHttpResponse response = null;
        CloseableHttpClient client = null;

        try {
            client = HttpClientBuilder.create().build();
            httpEntity.setEntity(new StringEntity(data.toJSONString(), ContentType.APPLICATION_JSON));

            response = client.execute(httpEntity);
            if (response != null && response.getStatusLine().getStatusCode() == 200) {
                BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(response.getEntity().getContent()));
                StringBuffer result = new StringBuffer();
                String line = "";
                while ((line = bufferedReader.readLine()) != null) {
                    result.append(line);
                }
                bufferedReader.close();
                return JSONObject.parseObject(result.toString());
            }

        } catch (ClientProtocolException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            try {
                if (response != null) {
                    response.close();
                }
                client.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return null;
    }

    @Autowired
    private ICTopOauthTokenService tokenService;
}
