package org.jeecg.modules.ctop.service.impl;

import cn.com.ctop.common.utils.PropertiesUtils;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.http.HttpEntity;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpEntityEnclosingRequestBase;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.ContentType;
import org.apache.http.entity.StringEntity;
import org.apache.http.entity.mime.MultipartEntityBuilder;
import org.apache.http.entity.mime.content.FileBody;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClientBuilder;
import org.jeecg.modules.ctop.entity.ByteDanceImageInfo;
import org.jeecg.modules.ctop.entity.ByteDanceVideoInfo;
import org.jeecg.modules.ctop.entity.CTopOauthToken;
import org.jeecg.modules.ctop.entity.FileInfo;
import org.jeecg.modules.ctop.mapper.ByteDanceImageInfoMapper;
import org.jeecg.modules.ctop.mapper.ByteDanceVideoInfoMapper;
import org.jeecg.modules.ctop.mapper.FileInfoMapper;
import org.jeecg.modules.ctop.service.ICTopOauthTokenService;
import org.jeecg.modules.ctop.service.IFileInfoService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.*;
import java.net.URI;
import java.util.HashMap;
import java.util.Map;

/**
 * @Description: 文件信息
 * @Author: jeecg-boot
 * @Date: 2019-07-28
 * @Version: V1.0
 */
@Service
public class FileInfoServiceImpl extends ServiceImpl<FileInfoMapper, FileInfo> implements IFileInfoService {
    private static final Logger logger = LoggerFactory.getLogger(FileInfoServiceImpl.class);

    @Override
    public Map<String, Object> uploadVideoToTemplate(String accountId, Long videoFileId, String templatename) {
        Map<String, Object> resultMap = new HashMap<>();
        FileInfo fileInfo = fileInfoMapper.selectById(videoFileId);
        if (null == fileInfo) {
            resultMap.put("message", "该视频文件信息不存在");
            resultMap.put("code", -1);
            return resultMap;
        }
        //TODO查询是否已经上传过头条平台
        CTopOauthToken token = tokenService.getOAuthTokenByAccountId(accountId);
        JSONObject resultObject = uploadAdvideo(token.getAccessToken(), fileInfo, token.getAccountId() + "");
        System.out.println(resultObject);
        Integer code = resultObject.getInteger("code");
        String message = resultObject.getString("message");
        if (null == code || code != 0) {
            logger.info("视频文件上传失败,accountId:{},message:{}", accountId, message);
            resultMap.put("code", -1);
            resultMap.put("message", "视频文件上传失败");
            return resultMap;
        }
        JSONObject data = resultObject.getJSONObject("data");
        ByteDanceVideoInfo videoInfo = new ByteDanceVideoInfo(data, token, videoFileId);
        videoInfoMapper.insert(videoInfo);
        resultMap.put("code", 0);
        resultMap.put("message", "视频文件上传成功");
        return resultMap;
    }

    @Autowired
    private ByteDanceVideoInfoMapper videoInfoMapper;
    @Autowired
    private ByteDanceImageInfoMapper imageInfoMapper;

    @Override
    public Map<String, Object> uploadImageToTemplate(String accountId, Long imageFileId, String templatename) {
        Map<String, Object> resultMap = new HashMap<>();
        FileInfo fileInfo = fileInfoMapper.selectById(imageFileId);
        if (null == fileInfo) {
            resultMap.put("message", "该图片文件信息不存在");
            resultMap.put("code", -1);
            return resultMap;
        }
        if (null == fileInfo.getType() || !"IMAGE".equals(fileInfo.getType())) {
            resultMap.put("message", "文件格式错误");
            resultMap.put("code", -1);
            return resultMap;
        }
        //TODO查询是否已经上传过头条平台
        CTopOauthToken token = tokenService.getOAuthTokenByAccountId(accountId);
        JSONObject resultObject = uploadAdImage(token.getAccessToken(), fileInfo, token.getAccountId() + "");
        System.out.println(resultObject);
        Integer code = resultObject.getInteger("code");
        String message = resultObject.getString("message");
        if (null == code || code != 0) {
            logger.info("图片文件上传失败,accountId:{},message:{}", accountId, message);
            resultMap.put("code", -1);
            resultMap.put("message", "图片文件上传失败");
            return resultMap;
        }
        JSONObject data = resultObject.getJSONObject("data");
        ByteDanceImageInfo imageInfo = new ByteDanceImageInfo(data, token, imageFileId);
        imageInfoMapper.insert(imageInfo);
        resultMap.put("code", 0);
        resultMap.put("message", "图片文件上传成功");
        return resultMap;
    }

    @Override
    public Map<String, Object> getIndustryList(String accountId) {
        Map<String, Object> resultMap = new HashMap<>();
        CTopOauthToken token = tokenService.getOAuthTokenByAccountId(accountId);
        JSONObject result = searchIndustry(token);
        resultMap.put("code", 0);
        resultMap.put("message", "数据获取成功");
        resultMap.put("data", result);
        return resultMap;
    }

    public JSONObject searchIndustry(CTopOauthToken token) {
        // 请求地址
        String url = "https://ad.toutiao.com/open_api/2/tools/industry/get/";

        // 构造请求
        HttpEntityEnclosingRequestBase httpEntity = new HttpEntityEnclosingRequestBase() {
            @Override
            public String getMethod() {
                return "GET";
            }
        };
        httpEntity.setHeader("Access-Token", token.getAccessToken());
        CloseableHttpResponse response = null;
        CloseableHttpClient client = null;
        try {
            client = HttpClientBuilder.create().build();
            httpEntity.setURI(URI.create(url));
            httpEntity.setEntity(new StringEntity(new JSONObject().toJSONString(), ContentType.APPLICATION_JSON));
            response = client.execute(httpEntity);
            if (response != null && response.getStatusLine().getStatusCode() == 200) {
                BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(response.getEntity().getContent()));
                StringBuffer result = new StringBuffer();
                String line = "";
                while ((line = bufferedReader.readLine()) != null) {
                    result.append(line);
                }
                bufferedReader.close();
                return JSONObject.parseObject(result.toString());
            }
        } catch (ClientProtocolException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            try {
                if (response != null) {
                    response.close();
                }
                client.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return null;
    }

    public JSONObject uploadAdImage(String accessToken, FileInfo fileInfo, String advertiserId) {
        // 请求地址
        String url = PropertiesUtils.getValue("bytedance_config", "bytedance_api_url") + PropertiesUtils.getValue("bytedance_config", "bytedance_v2_file_image_ad");
        // 构造请求
        HttpPost httpPost = new HttpPost(url);
        httpPost.setHeader("Access-Token", accessToken);
        // 文件参数
        FileBody file = new FileBody(new File(fileInfo.getPath()));
        MultipartEntityBuilder entityBuilder = MultipartEntityBuilder.create().addPart("image_file", file);
        // 其他参数
        entityBuilder.addTextBody("advertiser_id", advertiserId);
        try {
            entityBuilder.addTextBody("image_signature", DigestUtils.md5Hex(new FileInputStream(fileInfo.getPath())));
        } catch (IOException e) {
            e.printStackTrace();
        }
        HttpEntity entity = entityBuilder.build();
        CloseableHttpResponse response = null;
        CloseableHttpClient client = null;
        try {
            client = HttpClientBuilder.create().build();
            httpPost.setURI(URI.create(url));
            httpPost.setEntity(entity);
            response = client.execute(httpPost);
            if (response != null && response.getStatusLine().getStatusCode() == 200) {
                BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(response.getEntity().getContent()));
                StringBuffer result = new StringBuffer();
                String line = "";
                while ((line = bufferedReader.readLine()) != null) {
                    result.append(line);
                }
                bufferedReader.close();
                return JSONObject.parseObject(result.toString());
            }
        } catch (ClientProtocolException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            try {
                if (response != null) {
                    response.close();
                }
                client.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return null;
    }

    public JSONObject uploadAdvideo(String accessToken, FileInfo fileInfo, String advertiserId) {
        // 请求地址
        String url = PropertiesUtils.getValue("bytedance_config", "bytedance_api_url") + PropertiesUtils.getValue("bytedance_config", "bytedance_v2_file_video_ad");
        // 构造请求
        HttpPost httpPost = new HttpPost(url);
        httpPost.setHeader("Access-Token", accessToken);
        // 文件参数
        FileBody file = new FileBody(new File(fileInfo.getPath()));
        MultipartEntityBuilder entityBuilder = MultipartEntityBuilder.create().addPart("video_file", file);
        // 其他参数
        entityBuilder.addTextBody("advertiser_id", advertiserId);
        try {
            entityBuilder.addTextBody("video_signature", DigestUtils.md5Hex(new FileInputStream(fileInfo.getPath())));
        } catch (IOException e) {
            e.printStackTrace();
        }
        HttpEntity entity = entityBuilder.build();
        CloseableHttpResponse response = null;
        CloseableHttpClient client = null;
        try {
            client = HttpClientBuilder.create().build();
            httpPost.setURI(URI.create(url));
            httpPost.setEntity(entity);
            response = client.execute(httpPost);
            if (response != null && response.getStatusLine().getStatusCode() == 200) {
                BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(response.getEntity().getContent()));
                StringBuffer result = new StringBuffer();
                String line = "";
                while ((line = bufferedReader.readLine()) != null) {
                    result.append(line);
                }
                bufferedReader.close();
                return JSONObject.parseObject(result.toString());
            }
        } catch (ClientProtocolException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            try {
                if (response != null) {
                    response.close();
                }
                client.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return null;
    }

    @Autowired
    private FileInfoMapper fileInfoMapper;
    @Autowired
    private ICTopOauthTokenService tokenService;
}
